/*
 * BasePanel.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JViewport;
import javax.swing.ListSelectionModel;

import org.underworldlabs.jarsearch.JarSearchController;
import org.underworldlabs.jarsearch.JarSearchResult;
import org.underworldlabs.jarsearch.JarSearchWorker;
import org.underworldlabs.jarsearch.SystemResources;
import org.underworldlabs.swing.ActionPanel;
import org.underworldlabs.swing.CollapsibleTitledPanel;
import org.underworldlabs.swing.DynamicComboBoxModel;
import org.underworldlabs.swing.GUIUtils;
import org.underworldlabs.swing.actions.ActionUtilities;
import org.underworldlabs.swing.util.SwingWorker;
import org.underworldlabs.util.MiscUtils;
import org.underworldlabs.util.SystemProperties;

/** 
 * Base GUI panel and entry form.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class BasePanel extends ActionPanel 
                       implements JarSearchController {
    
    /** the class name field */
    private JComboBox classNameField;
    
    /** add path button */
    private JButton addPathButton;
    
    /** remove path button */
    private JButton removePathButton;

    /** move path up button */
    private JButton movePathUpButton;

    /** move path down button */
    private JButton movePathDownButton;

    /** remove path button */
    private JButton searchButton;

    /** remove path button */
    private JButton stopButton;

    /** the path list */
    private JList searchPathList;
    
    /** progress text area */
    private JTextArea progressText;

    /** the result table */
    private SearchResultsTable searchResultsTable;
    
    /** search results table scroller */
    private JScrollPane tableScroller;
    
    /** the path list model */
    private JarSearchListModel pathListModel;
    
    /** search worker object */
    private JarSearchWorker searchWorker;
    
    /** match case check box */
    private JCheckBox matchCaseCheck;

    /** whole words check box */
    private JCheckBox wholeWordsCheck;

    /** regex check box */
    private JCheckBox regexCheck;
    
    /** resources check box */
    private JCheckBox resourcesCheck;
    
    /** the serach query combo box model */
    private DynamicComboBoxModel searchQueryModel;
    
    /** Creates a new instance of BasePanel */
    public BasePanel() {
        super(new GridBagLayout());
        try {
            init();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void init() throws Exception {
        //Dimension fieldDim = new Dimension(1, 22);

        searchQueryModel = new DynamicComboBoxModel(
                SystemResources.getSearchQueryHistory());
        classNameField = new JComboBox(searchQueryModel);
        classNameField.setEditable(true);
        classNameField.setSelectedIndex(-1);
        
        classNameField.getEditor().getEditorComponent().addKeyListener(new ClassNameFieldKeyAdapter());
        
        //classNameField.setPreferredSize(fieldDim);
        
        searchButton = ActionUtilities.createButton(
                this, GUIUtilities.loadIcon("Zoom16.gif"), 
                GUIUtilities.getResourceString("BasePanel.searchButton"), "search");
        stopButton = ActionUtilities.createButton(
                this, GUIUtilities.loadIcon("Stop16.gif"), 
                GUIUtilities.getResourceString("BasePanel.stopButton"), "stop");
        stopButton.setEnabled(false);

        // path add/remove components
        addPathButton = ActionUtilities.createButton(this, 
                GUIUtilities.getResourceString("BasePanel.addPathButton"), "addPath");
        removePathButton = ActionUtilities.createButton(this, 
                GUIUtilities.getResourceString("BasePanel.removePathButton"), "removePath");

        movePathUpButton = ActionUtilities.createButton(this, 
                GUIUtilities.getResourceString("BasePanel.movePathUpButton"), "movePathUp");
        movePathDownButton = ActionUtilities.createButton(this, 
                GUIUtilities.getResourceString("BasePanel.movePathDownButton"), "movePathDown");
        
        JComponent[] c = {searchButton,
                          stopButton, 
                          addPathButton, 
                          removePathButton,
                          movePathUpButton,
                          movePathDownButton};
        
        Dimension buttonDim = new Dimension(150, 25);
        for (int i = 0; i < c.length; i++) {
            c[i].setPreferredSize(buttonDim);
        }

        searchPathList = new JList();
        searchPathList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        pathListModel = new JarSearchListModel();
        searchPathList.setModel(pathListModel);
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets.bottom = 5;
        gbc.insets.left = 5;
        gbc.insets.right = 5;
        gbc.anchor = GridBagConstraints.NORTHWEST;

        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = GridBagConstraints.REMAINDER;
        
        String title = GUIUtilities.getResourceString("BasePanel.pathPanelBorder");
        CollapsibleTitledPanel pathBasePanel = new CollapsibleTitledPanel(title);

        JPanel pathPanel = pathBasePanel.getContentPane();
        pathPanel.setLayout(new GridBagLayout());

        pathPanel.add(new JScrollPane(searchPathList), gbc);
        gbc.gridx = 1;
        gbc.gridheight = 1;
        gbc.gridwidth = 1;
        gbc.weightx = 0;
        gbc.weighty = 0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        pathPanel.add(addPathButton, gbc);
        gbc.gridy++;
        gbc.insets.top = 5;
        pathPanel.add(removePathButton, gbc);
        gbc.gridy++;
        pathPanel.add(movePathUpButton, gbc);
        gbc.gridy++;
        pathPanel.add(movePathDownButton, gbc);

        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.weightx = 0;
        gbc.gridwidth = 1;
        gbc.insets.top = 13;
        gbc.fill = GridBagConstraints.NONE;
        add(new JLabel(GUIUtilities.getResourceString("BasePanel.fileNameLabel")), gbc);
        gbc.gridx++;
        gbc.insets.left = 0;
        gbc.weightx = 1.0;
        gbc.insets.top = 10;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        add(classNameField, gbc);

        matchCaseCheck = new JCheckBox(
                GUIUtilities.getResourceString("BasePanel.matchCaseCheck"));
        matchCaseCheck.setSelected(
                SystemProperties.getBooleanProperty("user", "option.match-case"));
        
        wholeWordsCheck = new JCheckBox(
                GUIUtilities.getResourceString("BasePanel.wholeWordsCheck"));
        wholeWordsCheck.setSelected(
                SystemProperties.getBooleanProperty("user", "option.whole-words"));

        regexCheck = new JCheckBox(
                GUIUtilities.getResourceString("BasePanel.regexCheck"));
        regexCheck.setSelected(
                SystemProperties.getBooleanProperty("user", "option.regex"));

        resourcesCheck = new JCheckBox(
                GUIUtilities.getResourceString("BasePanel.resourceCheck"));
        resourcesCheck.setSelected(
                SystemProperties.getBooleanProperty("user", "option.resources"));
        
        title = GUIUtilities.getResourceString("BasePanel.optionsPanelBorder");
        CollapsibleTitledPanel optionsBasePanel = new CollapsibleTitledPanel(title);
        
        JPanel optionsPanel = optionsBasePanel.getContentPane();
//        optionsPanel.setLayout(new BoxLayout(optionsPanel, BoxLayout.Y_AXIS));
        optionsPanel.setLayout(new GridLayout(2, 2));
        optionsPanel.add(matchCaseCheck);
        optionsPanel.add(wholeWordsCheck);
        optionsPanel.add(regexCheck);
        optionsPanel.add(resourcesCheck);

        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 1.0;
        gbc.insets.top = 0;
        gbc.insets.left = 5;
        gbc.insets.bottom = 0;
        gbc.fill = GridBagConstraints.BOTH;
        add(optionsBasePanel, gbc);

        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 1.0;
        gbc.insets.top = 0;
        gbc.insets.left = 5;
        gbc.fill = GridBagConstraints.BOTH;
        add(pathBasePanel, gbc);
        
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 5, 5));
        buttonPanel.add(searchButton);
        buttonPanel.add(stopButton);

        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 0;
        gbc.insets.left = 5;
        gbc.insets.bottom = 5;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        add(buttonPanel, gbc);

        progressText = new JTextArea();
        progressText.setEditable(false);
        progressText.setMargin(new Insets(2,2,2,2));

        JPanel textPanel = new JPanel(new BorderLayout());
        textPanel.setPreferredSize(new Dimension(1, 100));
        textPanel.add(new JScrollPane(progressText), BorderLayout.CENTER);

        searchResultsTable = new SearchResultsTable();
        tableScroller = new JScrollPane(searchResultsTable);
        
        JSplitPane splitPane = new JSplitPane(
                JSplitPane.VERTICAL_SPLIT, textPanel, tableScroller);
        splitPane.setResizeWeight(0.35);
        splitPane.setOneTouchExpandable(true);

        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.insets.top = 5;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets.left = 5;
        add(splitPane, gbc);

        setPreferredSize(new Dimension(600, 580));
        optionsBasePanel.setCollapsed(true);
    }

    private boolean hasValidSearchParams() {
        String value = getSearchQuery();
        if (MiscUtils.isNull(value)) {
            GUIUtilities.displayErrorMessage(
                    GUIUtilities.getResourceString("BasePanel.nameInvalid"));
            return false;
        }

        if (pathListModel.getSize() == 0) {
            GUIUtilities.displayErrorMessage(
                    GUIUtilities.getResourceString("BasePanel.pathsInvalid"));
            return false;
        }
        
        return true;        
    }

    public void stop() {
        if (worker != null) {
            worker.interrupt();
        }
    }

    /** swing worker thread */
    private SwingWorker worker;

    public void search() {
        if (!hasValidSearchParams()) {
            return;
        }

        if (searchWorker == null) {
            searchWorker = new JarSearchWorker(this);
        }
        
        worker = new SwingWorker() {
            public Object construct() {
                BasePanel.this.start();
                doWork();
                return "done";
            }
            public void finished() {
                BasePanel.this.finished();
            }
        };
        worker.start();
    }

    /** Begins the search process. */
    private void doWork() {
        try {
            searchResultsTable.clear();
            appendProgress(null);
            searchWorker.search(getSearchQuery(),
                    pathListModel.getItems(), isMatchCaseSearch(), 
                    isRegexSearch(), isWholeWordsSearch(), isResourcesSearch());
        }
        catch (InterruptedException e) {
            appendProgress("\nProcess cancelled.");
        }
        catch (MalformedURLException e) {
            String message = GUIUtilities.getResourceString(
                    "BasePanel.errorLoadingJar") + e.getMessage();
            appendProgress(message);
            GUIUtilities.displayExceptionErrorDialog(message, e);
        }
        catch (IOException e) {
            String message = GUIUtilities.getResourceString(
                    "BasePanel.errorLoadingJar") + e.getMessage();
            appendProgress(message);
            GUIUtilities.displayExceptionErrorDialog(message, e);
        }
        finally {
            scrollResultsTableToEnd();
        }
    }
    
    /** the last absolute path selected */
    private String lastDirectorySelected;
    
    public void addPath() {
        JFileChooser fileChooser = new SearchPathFileChooser(lastDirectorySelected);
        int result = fileChooser.showOpenDialog(GUIUtilities.getApplicationFrame());
        File file = fileChooser.getSelectedFile();
        
        if (file == null || result == JFileChooser.CANCEL_OPTION) {
            return;
        }

        lastDirectorySelected = file.getAbsolutePath();
        pathListModel.addItem(lastDirectorySelected);
    }

    public void removePath() {
        int index = searchPathList.getSelectedIndex();
        if (index == -1) {
            return;
        }
        pathListModel.removeItemAt(index);
        
        int size = pathListModel.getSize();
        if (size > 0) {
            if (index > size - 1) {
                searchPathList.setSelectedIndex(size - 1);
            } else {
                searchPathList.setSelectedIndex(index);
            }
        }
        
    }
    
    public void movePathUp() {
        int index = searchPathList.getSelectedIndex();
        if (index == -1) {
            return;
        }
        pathListModel.moveIndexUp(index);

        if (index > 0) {
            searchPathList.setSelectedIndex(index - 1);
        }
        
    }
    
    public void movePathDown() {
        int index = searchPathList.getSelectedIndex();
        if (index == -1) {
            return;
        }
        pathListModel.moveIndexDown(index);
        
        int size = pathListModel.getSize();
        if (index < size - 1) {
            searchPathList.setSelectedIndex(index + 1);
        }

    }

    /**
     * Adds the specified path to the result list.
     *
     * @param result    the result to add
     */
    public void addSearchResult(final JarSearchResult item) {
        GUIUtils.invokeLater(new Runnable() {
            public void run() {
                searchResultsTable.addItem(item);
                scrollResultsTableToEnd();
            }
        });
    }
    
    private void scrollResultsTableToEnd() {
        JViewport viewport = tableScroller.getViewport();
        int maxX = viewport.getView().getWidth() - viewport.getWidth();
        int maxY = viewport.getView().getHeight() - viewport.getHeight();
        
        Point pt = viewport.getViewPosition();
        pt.x = Math.max(maxX, pt.x);
        pt.y = Math.max(maxY, pt.y);
        viewport.setViewPosition(pt);
    }
    
    /**
     * Appends the specified text the progress output pane.
     *
     * @param text  the text to append
     */
    public void appendProgress(final String text) {
        GUIUtils.invokeLater(new Runnable() {
            public void run() {
                if (text != null) {
                    progressText.append(text);
                }
                progressText.append("\n");
                progressText.setCaretPosition(progressText.getText().length());
            }
        });
    }

    /**
     * Whether to do a case match.
     *
     * @return  true | false
     */
    public boolean isMatchCaseSearch() {
        return matchCaseCheck.isSelected();
    }
    
    /**
     * Whether to do use the input search text as 
     * a regex string.
     *
     * @return  true | false
     */
    public boolean isRegexSearch() {
        return regexCheck.isSelected();
    }
    
    public boolean isResourcesSearch() {
        return resourcesCheck.isSelected();
    }

    /**
     * Whether to do a case match.
     *
     * @return  true | false
     */
    public boolean isWholeWordsSearch() {
        return wholeWordsCheck.isSelected();
    }

    /**
     * Resets all fields.
     */
    public void reset() {
        classNameField.setSelectedIndex(-1);//Text("");
        progressText.setText("");
        searchResultsTable.clear();
        pathListModel.clear();
    }
    
    /**
     * Indicates that the process has finished.
     */
    public void finished() {
        String duration = MiscUtils.formatDuration(
                System.currentTimeMillis() - startTime);

        repaint();
        searching = false;
        stopButton.setEnabled(false);
        searchButton.setEnabled(true);
        
        addPathButton.setEnabled(true);
        removePathButton.setEnabled(true);
        movePathUpButton.setEnabled(true);
        movePathDownButton.setEnabled(true);
        
        GUIUtils.invokeLater(new Runnable() {
            public void run() {
                searchQueryModel.setElements(
                        SystemResources.getSearchQueryHistory());
            }
        });

        GUIUtilities.stopProcessing();

        appendProgress("\nTotal JAR files scanned: " + 
                searchWorker.getJarFileCount());
        appendProgress("Total files found: " + 
                searchWorker.getFindCount());

        appendProgress("\nSearch duration: " + duration);
        appendProgress(null);

        GUIUtilities.setActivityText(
                GUIUtilities.getResourceString("BasePanel.finishStatus"));
        
        GUIUtilities.setStatusText(searchWorker.getFindCount() + " " +
                GUIUtilities.getResourceString("BasePanel.foundStatus"));
    }

    /**
     * Returns the current search query text.
     *
     * @return  the search query text
     */
    public String getSearchQuery() {
        return classNameField.getEditor().getItem().toString();
    }
    
    /** the process start time */
    private long startTime;
    
    /**
     * Indicates that the process has started.
     */
    public void start() {
        // save the options and the search 
        // query in the history file
        GUIUtils.invokeLater(new Runnable() {
            public void run() {
                SystemResources.addSearchQuery(getSearchQuery());
                SystemProperties.setBooleanProperty(
                        "user", "option.match-case", matchCaseCheck.isSelected());
                SystemProperties.setBooleanProperty(
                        "user", "option.whole-words", wholeWordsCheck.isSelected());
                SystemProperties.setBooleanProperty(
                        "user", "option.regex", regexCheck.isSelected());
                SystemProperties.setBooleanProperty(
                        "user", "option.resources", resourcesCheck.isSelected());
                SystemResources.saveUserProperties(
                        SystemProperties.getProperties("user"));
                
                Vector<String> paths = pathListModel.getItems();
                Vector<String> pathHistory = SystemResources.getSearchPathHistory();
                // make sure we don't already have it
                boolean valueAdded = false;
                for (int i = 0, n = paths.size(); i < n; i++) {
                    valueAdded = false;
                    String path = paths.get(i);
                    for (int j = 0, m = pathHistory.size(); j < m; j++) {
                        if (path.equals(pathHistory.get(j))) {
                            valueAdded = true;
                            pathHistory.remove(j);
                            pathHistory.add(0, path);
                            break;
                        }
                    }
                    
                    if (!valueAdded) {
                        pathHistory.add(0, path);
                    }
                    
                }
                SystemResources.saveSearchPathHistory(pathHistory);

            }
        });
        
        searching = true;
        progressText.setText("");
        searchButton.setEnabled(false);
        stopButton.setEnabled(true);
        
        addPathButton.setEnabled(false);
        removePathButton.setEnabled(false);
        movePathUpButton.setEnabled(false);
        movePathDownButton.setEnabled(false);

        GUIUtilities.startProcessing();
        GUIUtilities.setStatusText("");
        GUIUtilities.setActivityText(
                GUIUtilities.getResourceString("BasePanel.startStatus"));

        startTime = System.currentTimeMillis();
    }

    private boolean searching;
    
    private class ClassNameFieldKeyAdapter extends KeyAdapter {

        public void keyReleased(KeyEvent e) {
            
            if (searching) {
                return;
            }
            
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                search();
            }
            
        }

    }
    
    
    private class JarSearchListModel extends AbstractListModel {
        
        /** found jars collection */
        private Vector<String> listItems;
        
        public JarSearchListModel() {}
        
        /**
         * Clears the table of all data.
         */
        public void clear() {
            if (listItems == null) {
                return;
            }
            listItems.clear();
            fireContentsChanged(this, -1, -1);
        }

        /**
         * Returns all the list items within a Vector.
         *
         * @return the list items
         */
        public Vector<String> getItems() {
            return listItems;
        }
        
        /**
         * Removes the item at the specified index.
         *
         * @param index     the index to be dumped
         */
        public void removeItemAt(int index) {
            if (listItems == null) {
                return;
            }
            
            int size = listItems.size();
            if (size == 0 || (size - 1) < index) {
                return;
            }

            listItems.remove(index);
            fireContentsChanged(this, -1, -1);
        }
        
        /**
         * Moves the item at the specified index up.
         *
         * @param index     the index to be moved
         */
        public void moveIndexUp(int index) {
            if (index == 0 || listItems == null) {
                return;
            }

            int size = listItems.size();
            if (size == 0 || (size - 1) < index) {
                return;
            }

            String item = listItems.get(index);
            listItems.remove(index);
            listItems.insertElementAt(item, --index);
            fireContentsChanged(this, -1, -1);
        }
        
        /**
         * Moves the item at the specified index up.
         *
         * @param index     the index to be moved
         */
        public void moveIndexDown(int index) {
            if (listItems == null) {
                return;
            }

            int size = listItems.size();
            if (size == 0 || (size - 1) <= index) {
                return;
            }

            String item = listItems.get(index);
            listItems.remove(index);
            listItems.insertElementAt(item, ++index);
            fireContentsChanged(this, -1, -1);
        }
        
        /**
         * Adds the specified item to the list.
         *
         * @param item   the path to be added
         */
        public void addItem(String item) {
            if (listItems == null) {
                listItems = new Vector<String>();
            }
            listItems.add(item);
            int index = listItems.indexOf(item);
            fireContentsChanged(this, index, index);
        }
        
        /**
         * Returns the component at the specified index.
         *
         * @param      index   an index into this list
         * @return     the component at the specified index
         */
        public Object getElementAt(int index) {
            if (listItems == null) {
                return "";
            }
            return listItems.elementAt(index);
        }

        /**
         * Returns the number of components in this list.
         *
         * @return  the number of components in this list
         */
        public int getSize() {
            if (listItems == null) {
                return 0;
            }
            return listItems.size();
        }

        private static final long serialVersionUID = 1L;
        
    }

    private static final long serialVersionUID = 1L;
    
}

