/*
 * JarSearchFrame.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import org.underworldlabs.jarsearch.SystemResources;
import org.underworldlabs.util.MiscUtils;
import org.underworldlabs.util.SystemProperties;

/** 
 * Application frame.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class JarSearchFrame extends JFrame implements ComponentListener {
    
    private int lastX;
    private int lastY;
    private int lastWidth;
    private int lastHeight;

    /** Creates a new instance of JarSearchFrame */
    public JarSearchFrame() {
        String title = GUIUtilities.getResourceString("JarSearchFrame.title");
        setTitle(title);
        
        ImageIcon icon = GUIUtilities.loadIcon("JarsSearch48.png");
        setIconImage(icon.getImage());
        
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(GUIUtilities.getBasePanel(), BorderLayout.CENTER);
        getContentPane().add(GUIUtilities.getStatusBarPanel(), BorderLayout.SOUTH);
        setJMenuBar(GUIUtilities.getMainMenu());
        
        GUIUtilities.registerApplicationFrame(this);
        pack();

        if (SystemProperties.containsKey("user", "window.position")) {
            setSizeAndPosition();
        }
        else {
            // screen center
            Toolkit toolkit = Toolkit.getDefaultToolkit();
            Dimension screenSize = toolkit.getScreenSize();

            Dimension frameDim = getSize();
            if (frameDim.height > screenSize.height) {
                frameDim.height = screenSize.height;
            }
            if (frameDim.width > screenSize.width) {
                frameDim.width = screenSize.width;
            }
            setLocation((screenSize.width - frameDim.width) / 2,
                              (screenSize.height - frameDim.height) / 2);
        }

        addComponentListener(this);
        setVisible(true);

    }
    
    public void setSizeAndPosition(int x, int y, int width, int height) {
        lastX = x;
        lastY = y;
        lastWidth = width;
        lastHeight = height;
        setSize(width, height);
        setLocation(x, y);
    }

    public void setSizeAndPosition() {
        String value = SystemProperties.getStringProperty("user", "window.position");
        String[] values = MiscUtils.splitSeparatedValues(value, ",");
        
        int x = 0;
        int y = 0;
        int width = 0;
        int height = 0;
        
        int dimValue = 0;
        
        for (int i = 0; i < values.length; i++) {
            dimValue = Integer.parseInt(values[i]);
            switch (i) {
                case 0:
                    x = dimValue;
                case 1:
                    y = dimValue;
                case 2:
                    width = dimValue;
                case 3:
                    height = dimValue;addComponentListener(this);
            }
        }
        setSizeAndPosition(x, y, width, height);
    }

    private void savePosition() {        
        int x = getX();
        int y = getY();
        int width = getWidth();
        int height = getHeight();

        if (x == lastX && y == lastY && 
                width == lastWidth && height == lastHeight) {
            return;
        }

        lastX = x;
        lastY = y;
        lastWidth = width;
        lastHeight = height;

        StringBuffer sb = new StringBuffer();
        sb.append(x).
           append(',').
           append(y).
           append(',').
           append(width).
           append(',').
           append(height);
        SystemProperties.setStringProperty("user", "window.position", sb.toString());
        SystemResources.saveUserProperties(SystemProperties.getProperties("user"));
    }
    
    public void componentMoved(ComponentEvent e) {
        savePosition();
    }

    public void componentResized(ComponentEvent e) {
        savePosition();
    }

    public void componentHidden(ComponentEvent e) {}
    public void componentShown(ComponentEvent e) {}

    private static final long serialVersionUID = 1L;
}

