/*
 * MainMenu.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

/** 
 * Application menu bar.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class MainMenu extends JMenuBar 
                      implements ActionListener {
    
    /** File | clear fields menu item */
    private JMenuItem clearAllMenu;
    
    /** File | exit menu item */
    private JMenuItem exitMenu;

    /** help | about menu item */
    private JMenuItem aboutMenu;
    
    /** Creates a new instance of mainMenu */
    public MainMenu() {
        JMenu fileMenu = new JMenu(
                GUIUtilities.getResourceString("MainMenu.file"));
        
        clearAllMenu = new JMenuItem(
                GUIUtilities.getResourceString("MainMenu.fileReset"), 'R');
        clearAllMenu.addActionListener(this);
        clearAllMenu.setToolTipText(
                GUIUtilities.getResourceString("MainMenu.fileReset.tip"));

        exitMenu = new JMenuItem(
                GUIUtilities.getResourceString("MainMenu.fileExit"), 'x');
        exitMenu.addActionListener(this);

        fileMenu.add(clearAllMenu);
        fileMenu.addSeparator();
        fileMenu.add(exitMenu);

        JMenu helpMenu = new JMenu(
                GUIUtilities.getResourceString("MainMenu.help"));
        aboutMenu = new JMenuItem(
                GUIUtilities.getResourceString("MainMenu.helpAbout"), 'A');
        aboutMenu.addActionListener(this);
        helpMenu.add(aboutMenu);
        
        add(fileMenu);
        add(helpMenu);
    }
    
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == clearAllMenu) {
            GUIUtilities.getBasePanel().reset();
        }
        else if (source == exitMenu) {
            System.exit(0);
        }
        else if (source == aboutMenu) {
            new AboutDialog();
        }
    }

    private static final long serialVersionUID = 1L;
}
