/*
 * SearchPathFileChooser.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import org.underworldlabs.jarsearch.SystemResources;
import org.underworldlabs.swing.FileSelector;

/** 
 * File chooser dialog with history drop-down at the top.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class SearchPathFileChooser extends JFileChooser
                                   implements ItemListener {
    
    /** Creates a new instance of SearchPathFileChooser */
    public SearchPathFileChooser(String path) {
        super(path);
        addChoosableFileFilter(
                new FileSelector(new String[] {"zip"}, "ZIP files"));
        
        FileSelector jarFilter = new FileSelector(new String[] {"jar"}, "JAR files");
        addChoosableFileFilter(jarFilter);

        addChoosableFileFilter(
                new FileSelector(new String[] {"war"}, "WAR files"));
        addChoosableFileFilter(
                new FileSelector(new String[] {"ear"}, "EAR files"));

        setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        setMultiSelectionEnabled(false);
        setFileFilter(jarFilter);
    }
    
    /** path history combo box */
    private JComboBox historyCombo;
    
    private JPanel getHistoryPanel() {
        JPanel panel = new JPanel(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = new Insets(12,10,0,10);       
        panel.add(new JLabel(GUIUtilities.getResourceString(
                    "SearchPathFileChooser.historyLabel")), gbc);
        
        historyCombo = new JComboBox(
                SystemResources.getSearchPathHistory());
        gbc.gridx++;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        gbc.insets.top = 10;
        gbc.insets.left = 0;
        panel.add(historyCombo, gbc);
        
        historyCombo.addItemListener(this);
        historyCombo.setSelectedIndex(-1);
        return panel;
    }
    
    public void itemStateChanged(ItemEvent e) {
        Object item = historyCombo.getSelectedItem();
        if (item != null) {
            String path = item.toString();
            File file = new File(path);
            if (file.exists()) {
                setSelectedFile(file);
            }
        }
    }

    protected JDialog createDialog(Component parent) throws HeadlessException {
        Frame frame = parent instanceof Frame ? (Frame)parent :
            (Frame)SwingUtilities.getAncestorOfClass(Frame.class, parent);
            
        String title = getUI().getDialogTitle(this);

        JDialog dialog = new JDialog(frame, title, true);

        Container contentPane = dialog.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(getHistoryPanel(), BorderLayout.NORTH);
        contentPane.add(this, BorderLayout.CENTER);

        if (JDialog.isDefaultLookAndFeelDecorated()) {
            boolean supportsWindowDecorations =
            UIManager.getLookAndFeel().getSupportsWindowDecorations();

            if (supportsWindowDecorations) {
                dialog.getRootPane().setWindowDecorationStyle(
                                            JRootPane.FILE_CHOOSER_DIALOG);
            }

        }

        dialog.pack();

        // set location to the center of the desktop pane
        dialog.setLocation(GUIUtilities.getLocationForDialog(dialog.getSize()));
        return dialog;
    }

    private static final long serialVersionUID = 1L;
}

