package org.underworldlabs.jarsearch.gui;

import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

import org.underworldlabs.swing.actions.ReflectiveAction;

public class SearchResultsPopupMenu extends JPopupMenu implements MouseListener {
    
    private final JTable table;

    private Point lastSelectionPoint; 
    
    private ReflectiveAction reflectiveAction;

    public SearchResultsPopupMenu(JTable table) {

        this.table = table;
        table.addMouseListener(this);

        reflectiveAction = new ReflectiveAction(this);
        
        add(create("Copy selected file path", "copyCompletePathValue"));
        add(create("Copy selected file parent folder path", "copyParentPathValue"));
    }

    private JMenuItem create(String text, String actionCommand) {

        JMenuItem menuItem = new JMenuItem(reflectiveAction);
        menuItem.setActionCommand(actionCommand);
        menuItem.setText(text);

        return menuItem;
    }

    private Object valueAtPoint(Point point) {

        int row = table.rowAtPoint(point);
        int col = table.columnAtPoint(point); 

        // must be archive file column 
        col = 1;
        
        return table.getValueAt(row, col);
    }

    public void copyCompletePathValue(ActionEvent e) {
        
        Object value = valueAtPoint(lastSelectionPoint);
        if (value != null) {
        
            addToClipboard(value);
        }

    }

    public void copyParentPathValue(ActionEvent e) {
        
        Object value = valueAtPoint(lastSelectionPoint);
        if (value != null) {
        
            File file = new File(value.toString());
            addToClipboard(file.getParent());
        }

    }

    private void addToClipboard(Object value) {

        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        StringSelection stringSelection = new StringSelection(value.toString());
        clipboard.setContents(stringSelection, stringSelection);
    }
    
    public void mousePressed(MouseEvent e) {

        maybeShowPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
        
        maybeShowPopup(e);
    }

    private void maybeShowPopup(MouseEvent e) {

        if (e.isPopupTrigger()) {
            
            lastSelectionPoint = e.getPoint();            
            selectCellAtPoint(lastSelectionPoint);
            
            show(e.getComponent(), lastSelectionPoint.x, lastSelectionPoint.y);
        }

    }

    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void selectCellAtPoint(Point point) {
        
        int row = table.rowAtPoint(point);
        int col = table.columnAtPoint(point); 

        table.setColumnSelectionInterval(col, col);
        table.setRowSelectionInterval(row, row);
    }

}
